@@ -1,24 +1,24 @@
+from .account import AccountInfo
 from .earn import Earn
 from .invoice import LineItem, Invoice, InvoiceList
-from .keys import PublicKey, PrivateKey
 from .memo import AgoraMemo
 from .payment import Payment, ReadOnlyPayment
 from .result import EarnResult, BatchEarnResult
-from .transaction import TransactionData
+from .transaction import TransactionData, TransactionState
 from .transaction_type import TransactionType
 
 __all__ = [
+    'AccountInfo',
     'Earn',
     'LineItem',
     'Invoice',
     'InvoiceList',
-    'PublicKey',
-    'PrivateKey',
     'AgoraMemo',
     'Payment',
     'ReadOnlyPayment',
     'EarnResult',
     'BatchEarnResult',
     'TransactionData',
+    'TransactionState',
     'TransactionType',
 ]
